@@ -1,44 +1,29 @@
 import inspect
+import time
+from contextlib import contextmanager
 from functools import wraps
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Callable, Iterator, Optional, Tuple, Type
 
 from django.core.exceptions import ImproperlyConfigured
 from django.http.request import HttpRequest
-from ninja.types import DictStrAny
 
 from ninja_extra.controllers.response import ControllerResponse
-from ninja_extra.permissions import BasePermission
-from ninja_extra.permissions.base import OperandHolder
+from ninja_extra.logger import request_logger
+from ninja_extra.signals import route_context_finished, route_context_started
 
 from ...dependency_resolver import get_injector
+from .context import RouteContext
 
 if TYPE_CHECKING:
     from ...controllers import APIController, Route
 
 
 class RouteFunctionContext:
-    permission_classes: Union[
-        List[Type[BasePermission]], List[OperandHolder[Any]], List
-    ] = []
-    request: Optional[HttpRequest] = None
-    args: List[Any] = []
-    kwargs: DictStrAny = dict()
-
-    def __init__(self, **kwargs: Any) -> None:
-        self.permission_classes = kwargs.get("permission_classes", [])
-        self.request = kwargs.get("request")
-        self.args = kwargs.get("args", [])
-        self.kwargs = kwargs.get("kwargs", dict())
+    def __init__(
+        self, controller_instance: "APIController", **view_func_kwargs: Any
+    ) -> None:
+        self.controller_instance = controller_instance
+        self.view_func_kwargs = view_func_kwargs
 
 
 class RouteFunction(object):
@@ -52,7 +37,7 @@ class RouteFunction(object):
         self._resolve_api_func_signature_(self.as_view)
 
     def __call__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> Any:
-        return self.as_view(request=request, *args, **kwargs)
+        return self.as_view(*args, request=request, **kwargs)
 
     def _get_required_api_func_signature(self) -> Tuple:
         skip_parameters = ["self", "request"]
@@ -74,17 +59,14 @@ class RouteFunction(object):
 
     def get_view_function(self) -> Callable:
         def as_view(request: HttpRequest, *args: Any, **kwargs: Any) -> Any:
-            controller_instance = self._get_controller_instance(
+            with self._prep_controller_route_execution_context(
                 request, *args, **kwargs
-            )
-            controller_instance.check_permissions()
-            api_func_kwargs = dict(**kwargs)
-
-            if self.has_request_param:
-                api_func_kwargs.update(request=request)
-            return self._process_view_function_result(
-                self.route.view_func(controller_instance, *args, **api_func_kwargs)
-            )
+            ) as ctx:
+                ctx.controller_instance.check_permissions()
+                result = self.route.view_func(
+                    ctx.controller_instance, *args, **ctx.view_func_kwargs
+                )
+            return self._process_view_function_result(result)
 
         return as_view
 
@@ -98,31 +80,90 @@ class RouteFunction(object):
             return result.status_code, result.convert_to_schema()
         return result
 
-    def _get_controller_instance(
-        self, request: HttpRequest, *args: Any, **kwargs: Any
-    ) -> "APIController":
-
+    def _get_controller_instance(self) -> "APIController":
         injector = get_injector()
-        init_kwargs = self._get_controller_init_kwargs(request, *args, **kwargs)
         assert self.controller
-        controller_instance: "APIController" = injector.create_object(self.controller)
 
-        controller_instance.context = RouteFunctionContext(**init_kwargs)
+        controller_instance: "APIController" = injector.create_object(self.controller)
         return controller_instance
 
-    def _get_controller_init_kwargs(
+    def _get_controller_route_context(
         self, request: HttpRequest, *args: Any, **kwargs: Any
-    ) -> Dict[str, Any]:
+    ) -> RouteContext:
+
         if not self.controller:
             raise ImproperlyConfigured("Controller object is required")
 
-        return dict(
+        init_kwargs = dict(
             permission_classes=self.route.permissions
             or self.controller.permission_classes,
             request=request,
             kwargs=kwargs,
             args=args,
         )
+        context = RouteContext(**init_kwargs)
+        return context
+
+    def _log_action(
+        self,
+        logger: Callable[..., Any],
+        request: HttpRequest,
+        duration: Optional[float] = None,
+        ex: Optional[Exception] = None,
+        **kwargs: Any,
+    ) -> None:
+        try:
+            msg = (
+                f'"{request.method.upper()} - {self.controller.__name__}[{self.as_view.__name__}] {request.path}" '  # type: ignore
+                f"{duration if duration else str(ex)}"
+            )
+            logger(msg, **kwargs)
+        except (Exception,):
+            pass
+
+    @contextmanager
+    def _prep_controller_route_execution_context(
+        self, request: HttpRequest, *args: Any, **kwargs: Any
+    ) -> Iterator[RouteFunctionContext]:
+        start_time = time.time()
+        context = self._get_controller_route_context(request, *args, **kwargs)
+        # send route_context_started signal
+        route_context_started.send(RouteContext, route_context=context)
+
+        controller_instance = self._get_controller_instance()
+        controller_instance.context = context
+
+        api_func_kwargs = dict(**kwargs)
+        if self.has_request_param:
+            api_func_kwargs.update(request=request)
+        try:
+            yield RouteFunctionContext(
+                controller_instance=controller_instance, **api_func_kwargs
+            )
+            self._log_action(
+                request_logger.info,
+                request=request,
+                duration=time.time() - start_time,
+                extra={
+                    "request": request,
+                },
+                exc_info=None,
+            )
+        except Exception as ex:
+            self._log_action(
+                request_logger.error,
+                request=request,
+                ex=ex,
+                extra={
+                    "request": request,
+                },
+                exc_info=None,
+            )
+            raise ex
+        finally:
+            controller_instance.context = None
+            # send route_context_finished signal
+            route_context_finished.send(RouteContext, route_context=None)
 
     def __str__(self) -> str:
         return self.route.route_params.path
@@ -136,18 +177,14 @@ class RouteFunction(object):
 class AsyncRouteFunction(RouteFunction):
     def get_view_function(self) -> Callable:
         async def as_view(request: HttpRequest, *args: Any, **kwargs: Any) -> Any:
-            controller_instance = self._get_controller_instance(
+            with self._prep_controller_route_execution_context(
                 request, *args, **kwargs
-            )
-            controller_instance.check_permissions()
-
-            api_func_kwargs = dict(**kwargs)
-            if self.has_request_param:
-                api_func_kwargs.update(request=request)
-            result = await self.route.view_func(
-                controller_instance, *args, **api_func_kwargs
-            )
-            return self._process_view_function_result(result=result)
+            ) as context:
+                context.controller_instance.check_permissions()
+                result = await self.route.view_func(
+                    context.controller_instance, *args, **context.view_func_kwargs
+                )
+            return self._process_view_function_result(result)
 
         return as_view
 
@@ -157,4 +194,4 @@ class AsyncRouteFunction(RouteFunction):
         return f"<AsyncRouteFunction, controller: {self.controller.__name__}, path: {self.__str__()}>"
 
     async def __call__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> Any:
-        return await self.as_view(request=request, *args, **kwargs)
+        return await self.as_view(*args, request=request, **kwargs)
