@@ -1,13 +1,13 @@
-from typing import Any, List, Optional, Tuple, Union, cast
+from typing import Any, List, Optional, Tuple, Type, Union, cast
 
 from django.apps import apps
 from django.core.exceptions import ImproperlyConfigured
-from injector import Injector
+from injector import Injector, Module
 
 from ninja_extra.apps import NinjaExtraConfig
 from ninja_extra.shortcuts import fail_silently
 
-__all__ = ["service_resolver", "get_injector"]
+__all__ = ["service_resolver", "get_injector", "register_injector_modules"]
 
 
 def get_injector() -> Injector:
@@ -34,3 +34,11 @@ def service_resolver(*services: type) -> Union[Tuple[Any], Any]:
             services_resolved.append(injector.get(service))
         return tuple(services_resolved)
     return injector.get(services[0])
+
+
+def register_injector_modules(*modules: Union[Module, Type[Module]]) -> None:
+    for module in modules:
+        injector = get_injector()
+        if isinstance(module, type) and issubclass(module, Module):
+            module = module()
+        injector.binder.install(module)
