@@ -1,9 +1,8 @@
-from .base import SAFE_METHODS, BasePermission, BasePermissionMetaclass
+from .base import SAFE_METHODS, BasePermission
 from .common import AllowAny, IsAdminUser, IsAuthenticated, IsAuthenticatedOrReadOnly
 
 __all__ = [
     "BasePermission",
-    "BasePermissionMetaclass",
     "SAFE_METHODS",
     "AllowAny",
     "IsAuthenticated",
